/*
 * app.c
 *
 *  Created on: 24. 4. 2017
 *      Author: Martin
 */

#include <stm32l0xx_hal.h>

#include <temp.h>
#include <com.h>
#include <rtc.h>
#include <disp.h>

#define UPDATE_ADC_PERIOD_MS 1
#define UPDATE_VALUES_PERIOD_MS 250
#define UPDATE_DISP_PERIOD_MS 250
#define TOGGLE_CELL2_PERIOD_MS 30000

typedef enum
{
    MODE_1,
    MODE_2

} mode_e;

void switch_mode();
void string_received_callback(char *message);
void temps_on_line(char *string, float a, float b);

float temp_in;
float temp_out;
mode_e mode;
now_t real_time;
disp_t disp;
float maintained_temp;


void app_init()
{
    temp_init();

    // Watch serial line
    com_init();

    // Initialize RTC
    rtc_init();

    // Initialize lcd display
    disp_init();
}

void app_run()
{
    uint32_t tick_val_update = 0;
    uint32_t tick_disp_update = 0;
    uint32_t tick_cell2_toggle = 0;

    while (1)
    {
        // If values has not been updated of UPDATE_VALUES_PERIOD_MS ...
        if (HAL_GetTick() >= tick_val_update)
        {
            // ... update measured values of temperature and "real time"
            // adc_get(&temp_in, &temp_out);

            // TODO Start
            temp_get(&temp_in, &temp_out);
            rtc_get(&real_time);

            // Plan next update
            tick_val_update = HAL_GetTick() + UPDATE_VALUES_PERIOD_MS;
        }

        // If display has not been updated of DISP_UPDATE_PERIOD_MS ...
        if (HAL_GetTick() >= tick_disp_update)
        {
            // ... update it
            // __disable_irq();
            // sprintf(disp[0], "%5.2fC%5.2fC", temp_in, temp_out);
            // sprintf(disp[1], "%2d.%2d.%4d %2d:%2d", now.day, now.mon, now.year, now.hrs, now.min);
            temps_on_line(disp.image[0], temp_in, temp_out);
            sprintf(disp.image[1], "%2d.%02d.%4d %2d:%02d", real_time.day, real_time.mon, real_time.year, real_time.hrs, real_time.min);
            disp_update(&disp);
            // __enable_irq();

            // Plan next update
            tick_disp_update = HAL_GetTick() + UPDATE_DISP_PERIOD_MS;
        }

        switch (mode)
        {
        case MODE_1:
            HAL_GPIO_WritePin(GPIOA, GPIO_PIN_11, GPIO_PIN_RESET);
            HAL_GPIO_WritePin(GPIOA, GPIO_PIN_12, GPIO_PIN_RESET);
            maintained_temp = temp_in;
            break;
        case MODE_2:
            HAL_GPIO_WritePin(GPIOA, GPIO_PIN_11, GPIO_PIN_RESET);

            // If cell 2 has not been toggled for TOGGLE_CELL2_PERIOD_MS ...
            if (HAL_GetTick() >= tick_cell2_toggle)
            {
                // ... toggle it
                HAL_GPIO_TogglePin(GPIOA, GPIO_PIN_12);

                // Plan next toggle
                tick_cell2_toggle = HAL_GetTick() + TOGGLE_CELL2_PERIOD_MS;
            }
            break;
        default:
            break;
        }
    }
}

void switch_mode()
{
    mode++;
    if (mode > MODE_2)
    {
        mode = MODE_1;
    }
}

void string_received_callback(char *message)
{
    sscanf(message, "%i.%i.%i %i:%i",&real_time.day, &real_time.mon, &real_time.year, &real_time.hrs, &real_time.min);
    rtc_set(&real_time);
}


void temps_on_line(char *string, float a, float b)
{
    int x;
    char *p = string;

    a <= 0. ? a = 0. : 1 ;

    b <= 0. ? b = 0. : 1 ;

    x = a * 10;

    *p = (char)(x / 1000) + '0';
    x -= (*p - '0') * 1000;
    if((*p) == '0')
    {
        *p = ' ';
    }
    p++;

    *p = (char)(x / 100) + '0';
    x -= (*p - '0') * 100;
    p++;

    *p = (char)(x / 10) + '0';
    x -= (*p - '0') * 10;
    p++;

    *p = '.';
    p++;

    *p = (char)(x / 1) + '0';
    x -= (*p - '0') * 1;
    p++;

    *p = '';
    p++;

    *p = 'C';
    p++;

    *p = ' ';
    p++;

    *p = ' ';
    p++;

    x = b * 10;

    *p = (char)(x / 1000) + '0';
    x -= (*p - '0') * 1000;
    if((*p) == '0')
    {
        *p = ' ';
    }
    p++;

    *p = (char)(x / 100) + '0';
    x -= (*p - '0') * 100;
    p++;

    *p = (char)(x / 10) + '0';
    x -= (*p - '0') * 10;
    p++;

    *p = '.';
    p++;

    *p = (char)(x / 1) + '0';
    x -= (*p - '0') * 1;
    p++;

    *p = '';
    p++;

    *p = 'C';
    p++;
}
