/*
 * rtc.c
 *
 *  Created on: 24. 4. 2017
 *      Author: Martin
 */

#include <rtc.h>
#include <stm32l0xx_hal.h>

extern I2C_HandleTypeDef hi2c1;

void rtc_init()
{
    rtc_start();
}

void rtc_start()
{
    uint8_t temp;

    HAL_I2C_Mem_Read(&hi2c1, 0xd0, 0x00, 1, &temp, 1, 10);

    temp &= ~0x80;
    HAL_I2C_Mem_Write(&hi2c1, 0xd0, 0x00, 1, &temp, 1, 10);
}

void rtc_stop()
{
    uint8_t temp;

    HAL_I2C_Mem_Read(&hi2c1, 0xd0, 0x00, 1, &temp, 1, 10);

    temp |= 0x80;
    HAL_I2C_Mem_Write(&hi2c1, 0xd0, 0x00, 1, &temp, 1, 10);
}

void rtc_get(now_t *now)
{
    uint8_t buffer[8];

    HAL_I2C_Mem_Read(&hi2c1, 0xd0, 0x00, 1, buffer, 8, 10);

    now->min = ((buffer[1] >> 4) * 10) + (buffer[1]& 0x0f);
    now->hrs = ((buffer[2] >> 4) * 10) + (buffer[2]& 0x0f);
    now->day = ((buffer[4] >> 4) * 10) + (buffer[4]& 0x0f);
    now->mon = ((buffer[5] >> 4) * 10) + (buffer[5]& 0x0f);
    now->year = ((buffer[6] >> 4) * 10) + (buffer[6]& 0x0f) + 2000;
}

void rtc_set(now_t *now)
{
    uint8_t buffer[8];
    HAL_StatusTypeDef r;

    r = HAL_I2C_Mem_Read(&hi2c1, 0xd0, 0x00, 1, buffer, 1, 10);

    buffer[1] = ((now->min / 10) << 4) | (now->min % 10);
    buffer[2] = ((now->hrs / 10) << 4) | (now->hrs % 10);
    buffer[3] = 0x00;
    buffer[4] = ((now->day / 10) << 4) | (now->day % 10);
    buffer[5] = ((now->mon / 10) << 4) | (now->mon % 10);
    now->year %= 2000;
    buffer[6] = ((now->year / 10) << 4) | (now->year % 10);

    r = HAL_I2C_Mem_Write(&hi2c1, 0xd0, 0x00, 1, buffer, 8, 10);
}
